;	======================================
;	       Scheinwerfer mit Tiny12
;	======================================

;	------------- Port usage -------------

;	B0: Output Clock
;	B1: Output Load
;	B2: Output Data
;	B3: Input LampSelect
;	B4: Input Brightness

;	------------- Timer usage ------------
;	Timer0: Interrupt every 0,0068 s (Prescaler CK/256)

;	-- Standard Definitions for ATtiny12 -
	.include "tn12def.inc"

;	------------- Constants --------------
	.equ	cwaitl	=147		;value long waiting loop (1s)
	.equ	cwaits	=20		;value short waiting loop
	.equ	maxdata	=9		;number of data blocks
	.equ	ndriver	=1		;number of driver chips
	.equ	port	=portb
	.equ	pin	=pinb
	.equ	ddr	=ddrb
	.equ	clk	=0
	.equ	load	=1
	.equ	data	=2
	.equ	ilamp	=3
	.equ	ibrigh	=4

;	-------- Register definitions --------
	.def	stor	=r0		;storage variable for LPM
	.def	flags	=r1		;flags storage register
	.def	mlamp	=r12
	.def	mbrigh	=r13
	.def	temp3	=r14		;temporary variable 3
	.def	temp4	=r15		;temporary variable 4
	.def	temp	=r16		;temporary variable
	.def	temp1	=r17		;temporary variable 1
	.def	temp2	=r18		;temporary variable 2
	.def	waitco	=r19		;wait loop counter
	.def	cvold	=r20		;Vergleich alt
	.def	loop	=r21		;loop counter
	.def	loop2	=r22		;loop counter 2
	.def	loop3	=r23		;loop counter 3
	.def	proco	=r24		;programmcounter

;	- Program beginn, Interrupt Vectors --
.cseg
.org	0x0000

reset:	rjmp	initia			;Programmstart
exint0:	reti
pncnge:	reti
t0ovfl:	rjmp	timer0
eerdy:	reti
ancomp:	reti
t0coma:	reti
t0comb:	reti
wdtto:	reti
adccc:	reti

;	Initialisierung von RAM, I/O und Interrupts

initia:	cli				;Clear Interrupts
	ldi	temp,0x27
	out	osccal,temp		;Callibrate Oscillator (Tiny13: auto)
	ldi	temp,0x07
	out	ddr,temp		;Inputs & Outputs
	ldi	temp,0x18
	out	port,temp		;Set Pull-Ups

	ldi	temp,1<<toie0
	out	timsk,temp		;Timer0 Interrupt
	ldi	temp,(1<<cs01)
	out	tccr0,temp		;Timer0 Overflow, prescaler 256
	sei				;start interrupts

	ldi	zl,low(dini1*2)
	ldi	zh,high(dini1*2)
	rcall	cwrite

	ldi	zl,low(dini2*2)
	ldi	zh,high(dini2*2)
	rcall	cwrite

	ldi	zl,low(dini3*2)
	ldi	zh,high(dini3*2)
	rcall	cwrite

	ldi	zl,low(dini4*2)
	ldi	zh,high(dini4*2)
	rcall	cwrite

	ldi	zl,low(dini5*2)
	ldi	zh,high(dini5*2)
	rcall	cwrite

	ldi	temp,low(dbrigh)	;Restore Brightness
	rcall	eeread
	mov	mbrigh,temp2
	ldi	temp,0b00001010
	mov	temp3,temp
	mov	temp4,temp2
	rcall	cwra

	clr	mlamp

main1:	sbic	pin,ibrigh
	rjmp	main2
	rcall	waits
main1a:	sbis	pin,ibrigh
	rjmp	main1a
	rcall	waits
	ldi	temp2,5
	add	temp2,mbrigh	
	cpi	temp2,16
	brlo	main1b
	ldi	temp2,0
main1b:	mov	mbrigh,temp2
	ldi	temp,0b00001010
	mov	temp3,temp
	mov	temp4,temp2
	rcall	cwra
	ldi	temp,low(dbrigh)
	mov	temp2,mbrigh
	rcall	eewrit		;Save new brightness

main2:	sbic	pin,ilamp
	rjmp	main2b
	rcall	waits
main2a:	sbis	pin,ilamp
	rjmp	main2a
	rcall	waits
	inc	mlamp
main2b:	mov	temp,mlamp
	cpi	temp,0
	breq	seq10
	cpi	temp,1
	brne	main2c
	rjmp	seq20
main2c:	cpi	temp,2
	breq	seq11
	cpi	temp,3
	breq	seq12
	cpi	temp,4
	breq	seq19
	cpi	temp,5
	breq	yello
	cpi	temp,6
	breq	green
	cpi	temp,7
	breq	flash
	clr	mlamp
	rjmp	main1

seq10:	inc	proco
	cpi	proco,1
	breq	seq11
	cpi	proco,2
	breq	seq12
	cpi	proco,3
	breq	seq13
	cpi	proco,4
	breq	seq14
	cpi	proco,5
	breq	seq15
	cpi	proco,6
	breq	seq16
	cpi	proco,7
	breq	seq17
	cpi	proco,8
	breq	seq18
	cpi	proco,9
	breq	seq19
	clr	proco
seq10e:	rjmp	main1

seq11:	ldi	zl,low(data11*2)
	ldi	zh,high(data11*2)
	rjmp	maint

seq12:	ldi	zl,low(data12*2)
	ldi	zh,high(data12*2)
	rjmp	maint

seq13:	ldi	zl,low(data13*2)
	ldi	zh,high(data13*2)
	rjmp	maint

seq14:	ldi	zl,low(data14*2)
	ldi	zh,high(data14*2)
	rjmp	maint

seq15:	ldi	zl,low(data15*2)
	ldi	zh,high(data15*2)
	rjmp	maint

seq16:	ldi	zl,low(data16*2)
	ldi	zh,high(data16*2)
	rjmp	maint

seq17:	ldi	zl,low(data17*2)
	ldi	zh,high(data17*2)
	rjmp	maint

seq18:	ldi	zl,low(data18*2)
	ldi	zh,high(data18*2)
	rjmp	maint

seq19:	ldi	zl,low(data19*2)
	ldi	zh,high(data19*2)
	rjmp	maint

yello:	ldi	zl,low(dataye*2)
	ldi	zh,high(dataye*2)
	rjmp	maint

green:	ldi	zl,low(datagr*2)
	ldi	zh,high(datagr*2)
	rjmp	maint
	
flash:	ldi	zl,low(datafl*2)
	ldi	zh,high(datafl*2)
	rjmp	maint

seq20:	inc	proco
	cpi	proco,1
	breq	seq21
	cpi	proco,2
	breq	seq22
	cpi	proco,3
	breq	seq23
	cpi	proco,4
	breq	seq24
	cpi	proco,5
	breq	seq25
	cpi	proco,6
	breq	seq26
	cpi	proco,7
	breq	seq27
	cpi	proco,8
	breq	seq28
	cpi	proco,9
	breq	seq29
	cpi	proco,10
	breq	seq2a
	cpi	proco,11
	breq	seq2b
	cpi	proco,12
	breq	seq2c
	clr	proco
seq20e:	rjmp	main1

seq21:	ldi	zl,low(data21*2)
	ldi	zh,high(data21*2)
	rjmp	maint

seq22:	ldi	zl,low(data22*2)
	ldi	zh,high(data22*2)
	rjmp	maint

seq23:	ldi	zl,low(data23*2)
	ldi	zh,high(data23*2)
	rjmp	maint

seq24:	ldi	zl,low(data24*2)
	ldi	zh,high(data24*2)
	rjmp	maint

seq25:	ldi	zl,low(data25*2)
	ldi	zh,high(data25*2)
	rjmp	maint

seq26:	ldi	zl,low(data26*2)
	ldi	zh,high(data26*2)
	rjmp	maint

seq27:	ldi	zl,low(data27*2)
	ldi	zh,high(data27*2)
	rjmp	maint

seq28:	ldi	zl,low(data28*2)
	ldi	zh,high(data28*2)
	rjmp	maint

seq29:	ldi	zl,low(data29*2)
	ldi	zh,high(data29*2)
	rjmp	maint

seq2a:	ldi	zl,low(data2a*2)
	ldi	zh,high(data2a*2)
	rjmp	maint

seq2b:	ldi	zl,low(data2b*2)
	ldi	zh,high(data2b*2)
	rjmp	maint

seq2c:	ldi	zl,low(data2c*2)
	ldi	zh,high(data2c*2)

maint:	rcall	dwrite
	rcall	waitl
	rjmp	main1
	
;	--------- Long Waiting loop ----------
waitl:	clr	waitco
waitl1:	sbis	pin,ilamp
	rjmp	waitle
	sbis	pin,ibrigh
	rjmp	waitle
	cp	waitco,stor
	brlo	waitl1
waitle:	ret

;	--------- Short Waiting loop ---------
waits:	clr	waitco
waits2:	cpi	waitco,cwaits
	brlo	waits2
	ret

;	----- Write commando to Display ------
cwrite:	lpm
	mov	temp3,stor
	ldi	temp,1
	add 	zl,temp
	ldi	temp,0
	adc	zh,temp
	lpm
	mov	temp4,stor
	
cwra:	clr	loop2 ;Alternate writing

cwr0:	mov	temp1,temp3
	mov	temp2,temp4
	clr	loop
	cbi	port,clk
	cbi	port,load

cwr1:	cbi	port,data
	sbrc	temp1,7
	sbi	port,data
	rol	temp1
	rjmp	cwend

cwr2:	cbi	port,data
	sbrc	temp2,7
	sbi	port,data
	rol	temp2

cwend:	sbi	port,clk
	nop
	cbi	port,clk
	inc	loop
	cpi	loop,8
	brlo	cwr1
	cpi	loop,16
	brlo	cwr2

	inc	loop2
	cpi	loop2,ndriver
	brlo	cwr0

	rcall	wload
	ret

;	--------- Write full Display ---------
dwrite:	clr	loop3
	inc	loop3

dwr:	clr	loop2

dwr0:	mov	temp3,loop3
	lpm
	mov	temp4,stor
	ldi	temp,1
	add 	zl,temp
	ldi	temp,0
	adc	zh,temp
	mov	temp1,temp3
	mov	temp2,temp4
	clr	loop
	cbi	port,clk
	cbi	port,load

dwr1:	cbi	port,data
	sbrc	temp1,7
	sbi	port,data
	rol	temp1
	rjmp	dwend

dwr2:	cbi	port,data
	sbrc	temp2,7
	sbi	port,data
	rol	temp2

dwend:	sbi	port,clk
	nop
	cbi	port,clk
	inc	loop
	cpi	loop,8
	brlo	dwr1
	cpi	loop,16
	brlo	dwr2

	inc	loop2
	cpi	loop2,ndriver
	brlo	dwr0

	rcall	wload
	inc	loop3
	cpi	loop3,8
	brlo	dwr

	lpm
	ret

;	------- Load Display Registers -------
wload:	sbi	port,load
	nop
	cbi	port,load
	ret

;	------------ Read EEPROM -------------
eeread:	cli
	sbic	eecr,eewe
	rjmp	eeread
	out	eear,temp
	sbi	eecr,eere
	in	temp2,eedr
	sei
	ret

;	------------ Write EEPROM ------------
eewrit:	cli
	sbic	eecr,eewe
	rjmp	eewrit
	out	eear,temp
	out	eedr,temp2
	sbi	eecr,eemwe
	sbi	eecr,eewe
	sei
	ret

;	------------- Timer0 ISR -------------
timer0:	in	flags,sreg	;save flags
	inc	waitco
	out	sreg,flags	;restore flags
	reti

;DisplayCommands  M                 L
dini1:	.db	0b00001001,0b00000000; DecodeMode
dini2:	.db	0b00001010,0b00000000; Intensity
dini3:	.db	0b00001011,0b00000110; ScanLimit
dini4:	.db	0b00001100,0b00000001; Shutdown
dini5:	.db	0b00001111,0b00000000; DisplayTest

;Lampdata         76543210   76543210   76543210   76543210   76543210   76543210   76543210
data11:	.db	0b00001001,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b10010000,150
	
data12:	.db	0b10010110,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b01101001,150

data13:	.db	0b01100000,0b00001001,0b00000000,0b00000000,0b00000000,0b10010000,0b00000110,150

data14:	.db	0b00000000,0b10010110,0b00000000,0b00000000,0b00000000,0b01101001,0b00000000,150

data15:	.db	0b00000000,0b01100000,0b00001001,0b00000000,0b10010000,0b00000110,0b00000000,150

data16:	.db	0b00000000,0b00000000,0b10010110,0b00000000,0b01101001,0b00000000,0b00000000,150

data17:	.db	0b00000000,0b00000000,0b01100000,0b00001001,0b00000110,0b00000000,0b00000000,150

data18:	.db	0b00000000,0b00000000,0b00000000,0b01100110,0b00000000,0b00000000,0b00000000,150

data19:	.db	0b00000000,0b00000000,0b00000000,0b10010000,0b00000000,0b00000000,0b00000000,150

;Lampdata         76543210   76543210   76543210   76543210   76543210   76543210   76543210
data21:	.db	0b00000000,0b00010001,0b00010001,0b00000001,0b00010001,0b00010001,0b00000000,150
	
data22:	.db	0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,50

data23:	.db	0b00100000,0b00100010,0b00100010,0b00000010,0b00100010,0b00100010,0b00000010,150

data24:	.db	0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,50

data25:	.db	0b01000000,0b01000100,0b01000100,0b00000100,0b01000100,0b01000100,0b00000100,150

data26:	.db	0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,50

data27:	.db	0b00000000,0b10001000,0b10001000,0b00001000,0b10001000,0b10001000,0b00000000,150

data28:	.db	0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,50

data29:	.db	0b01000000,0b01000100,0b01000100,0b00000100,0b01000100,0b01000100,0b00000100,150

data2a:	.db	0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,50

data2b:	.db	0b00100000,0b00100010,0b00100010,0b00000010,0b00100010,0b00100010,0b00000010,150

data2c:	.db	0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,0b00000000,50

;Lampdata         76543210   76543210   76543210   76543210   76543210   76543210   76543210
dataye:	.db	0b00000000,0b00000000,0b00000000,0b00100000,0b00000000,0b00000000,0b00000000,10

datagr:	.db	0b00000000,0b00000000,0b00000000,0b01000000,0b00000000,0b00000000,0b00000000,10

datafl:	.db	0b01100000,0b11111111,0b11111111,0b10011111,0b11111111,0b11111111,0b00000110,10

;	------------ EEPROM Data -------------
.eseg

dbrigh:	.db	0